// Program generates header and source files for given description file.

// Program requires boost library (builded using version 1.54.0.1)
// Compile by typing in terminal: g++ -std=c++11 feature-headers-generator.cpp
// Example of running program (also has to be entered in terminal): ./a.out features-headers.csv

#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <vector>
#include <boost/algorithm/string.hpp>

using namespace std;

#define HEADER_FILE_NAME "feature-header-names.h"		// change this if you want different header file name
#define SOURCE_FILE_NAME "feature-header-names.cpp"	// change this if you want different source file name

void warning(ofstream& output) {
	output << "// author: Gorana Gojic" << endl << endl;
	output << "// This file is generated. All manually made changes will be overwritten at the next start of the generator." << endl;
	output << "// Please, change feature description file instead of directly changing this file." << endl << endl;
}

int main(int argc, char *argv[]) {

	if ( argc != 2 ) {
		cout << "Usage : " << argv[0] << " <input_filename>" << endl;
		return 1;
	}

	ifstream input;
	input.open(argv[1]);
	if ( !input.is_open() )
		return 2;

	string line;
	ofstream header_output, source_output;

	header_output.open( HEADER_FILE_NAME );
	if ( !header_output.is_open() )
		return 2;

	source_output.open( SOURCE_FILE_NAME );
	if ( !source_output.is_open() )
		return 2;

	warning(header_output);
	header_output << "#ifndef __FEATURE_HEADER_NAMES_H__\n#define __FEATURE_HEADER_NAMES_H__\n" << endl;
	warning(source_output);
	source_output << "#include <iostream>\n\n#include \"feature-header-names.h\"\n\nusing namespace std;" << endl;
	source_output << "void printHeader(string delim) {\n\tcout";

	int counter = 0;
	while ( getline(input, line) ) {
		if (line[0] == '#') continue;		// skip comment

		vector<string> parts;
		boost::split(parts, line, boost::is_any_of(","));
		for (auto i = 0; i < 3; i++)
			boost::algorithm::trim(parts[i]);

		// variable name
		header_output << "#define " << parts[0] << "\t\t\t";
		source_output << "\t\t<< " << parts[0] << " << delim\n";
		// string value
		header_output << "\"" << parts[1] << "\"\t\t\t";
		// comment
		header_output << "//  " << parts[2] << "\t\t\t\n";
	}
	header_output << "\n\n#endif // __FEATURE_HEADER_NAMES_H__";
	header_output.close();
	source_output << "\t<< endl; \n}\n" << endl;
	source_output << "int main(int argv, const char *args[]) {\n\tif ( argv != 2 ) {" << endl;
	source_output << "\t\tcout << \"Usage : \" << args[0] << \" <csv_delimiter>\" << endl;\n\t\treturn 1;\n\t}\n" << endl;
	source_output << "\tprintHeader(args[1]);\n\n\treturn 0;\n}" << endl;
	source_output.close();

	return 0;
}
